@@ -909,6 +909,7 @@ int add_file(struct thread_data *td, const char *fname)
 	}
 
 	f->fd = -1;
+	fio_file_reset(f);
 
 	if (td->files_size <= td->files_index) {
 		int new_size = td->o.nr_files + 1;
@@ -962,6 +963,19 @@ int add_file(struct thread_data *td, const char *fname)
 	return cur_files;
 }
 
+int add_file_exclusive(struct thread_data *td, const char *fname)
+{
+	struct fio_file *f;
+	unsigned int i;
+
+	for_each_file(td, f, i) {
+		if (!strcmp(f->file_name, fname))
+			return i;
+	}
+
+	return add_file(td, fname);
+}
+
 void get_file(struct fio_file *f)
 {
 	dprint(FD_FILE, "get file %s, ref=%d\n", f->file_name, f->references);
@@ -1136,6 +1150,7 @@ void dup_files(struct thread_data *td, struct thread_data *org)
 			assert(0);
 		}
 		__f->fd = -1;
+		fio_file_reset(__f);
 
 		if (f->file_name) {
 			__f->file_name = smalloc_strdup(f->file_name);
