@@ -22,7 +22,7 @@
 
 #include "lib/getopt.h"
 
-static char fio_version_string[] = "fio 1.43";
+static char fio_version_string[] = "fio 1.43.2";
 
 #define FIO_RANDSEED		(0xb1899bedUL)
 
@@ -194,6 +194,8 @@ static int __setup_rate(struct thread_data *td, enum fio_ddir ddir)
 	unsigned int bs = td->o.min_bs[ddir];
 	unsigned long long bytes_per_sec;
 
+	assert(ddir_rw(ddir));
+
 	if (td->o.rate[ddir])
 		bytes_per_sec = td->o.rate[ddir];
 	else
