@@ -21,9 +21,9 @@ void queue_io_piece(struct thread_data *td, struct io_piece *ipo)
 void log_io_u(struct thread_data *td, struct io_u *io_u)
 {
 	const char *act[] = { "read", "write", "sync", "datasync",
-				"sync_file_range" };
+				"sync_file_range", "wait", "trim" };
 
-	assert(io_u->ddir <= 4);
+	assert(io_u->ddir <= 6);
 
 	if (!td->o.write_iolog_file)
 		return;
@@ -133,7 +133,8 @@ int read_iolog_get(struct thread_data *td, struct io_u *io_u)
 			get_file(io_u->file);
 			dprint(FD_IO, "iolog: get %llu/%lu/%s\n", io_u->offset,
 						io_u->buflen, io_u->file->file_name);
-			if (ipo->delay) iolog_delay(td, ipo->delay);
+			if (ipo->delay)
+				iolog_delay(td, ipo->delay);
 		} else {
 			elapsed = mtime_since_genesis();
 			if (ipo->delay > elapsed)
@@ -143,7 +144,7 @@ int read_iolog_get(struct thread_data *td, struct io_u *io_u)
 
 		free(ipo);
 		
-		if (ipo->ddir != DDIR_WAIT)
+		if (io_u->ddir != DDIR_WAIT)
 			return 0;
 	}
 
@@ -293,6 +294,8 @@ static int read_iolog2(struct thread_data *td, FILE *f)
 				rw = DDIR_SYNC;
 			else if (!strcmp(act, "datasync"))
 				rw = DDIR_DATASYNC;
+			else if (!strcmp(act, "trim"))
+				rw = DDIR_TRIM;
 			else {
 				log_err("fio: bad iolog file action: %s\n",
 									act);
