@@ -169,6 +169,9 @@ static unsigned long long get_mult_bytes(const char *str, int len, void *data)
 {
 	const char *p;
 
+	if (len < 2)
+		return __get_mult_bytes(str, data);
+
 	/*
 	 * if the last char is 'b' or 'B', the user likely used
 	 * "1gb" instead of just "1g". If the second to last is also
@@ -278,6 +281,17 @@ static int check_int(const char *p, int *val)
 	return 1;
 }
 
+static int opt_len(const char *str)
+{
+	char *postfix;
+
+	postfix = strchr(str, ':');
+	if (!postfix)
+		return strlen(str);
+
+	return (int)(postfix - str);
+}
+
 #define val_store(ptr, val, off, or, data)		\
 	do {						\
 		ptr = td_var((data), (off));		\
@@ -320,7 +334,7 @@ static int __handle_option(struct fio_option *o, const char *ptr, void *data,
 			if (!vp->ival || vp->ival[0] == '\0')
 				continue;
 			all_skipped = 0;
-			if (!strncmp(vp->ival, ptr, strlen(ptr))) {
+			if (!strncmp(vp->ival, ptr, opt_len(ptr))) {
 				ret = 0;
 				if (o->roff1) {
 					if (vp->or)
